@@ -1,8 +1,12 @@
 import argparse
+import atexit
 import glob
 import json
+import logging
 import os
+import sys
 from dataclasses import dataclass, field, fields
+from datetime import datetime
 from enum import Enum, StrEnum
 from pathlib import Path
 from typing import Any, Optional, Sequence, Union
@@ -11,11 +15,20 @@ import shtab
 
 from vectorcode import __version__
 
+logger = logging.getLogger(name=__name__)
+
+
 PathLike = Union[str, Path]
 
 GLOBAL_CONFIG_PATH = os.path.join(
     os.path.expanduser("~"), ".config", "vectorcode", "config.json"
 )
+GLOBAL_INCLUDE_SPEC = os.path.join(
+    os.path.expanduser("~"), ".config", "vectorcode", "vectorcode.include"
+)
+GLOBAL_EXCLUDE_SPEC = os.path.join(
+    os.path.expanduser("~"), ".config", "vectorcode", "vectorcode.exclude"
+)
 
 CHECK_OPTIONS = ["config"]
 
@@ -23,6 +36,7 @@ CHECK_OPTIONS = ["config"]
 class QueryInclude(StrEnum):
     path = "path"
     document = "document"
+    chunk = "chunk"
 
     def to_header(self) -> str:
         """
@@ -44,12 +58,14 @@ class CliAction(Enum):
     update = "update"
     clean = "clean"
     prompts = "prompts"
+    chunks = "chunks"
 
 
 @dataclass
 class Config:
     no_stderr: bool = False
     recursive: bool = False
+    include_hidden: bool = False
     to_be_deleted: list[str] = field(default_factory=list)
     pipe: bool = False
     action: Optional[CliAction] = None
@@ -63,25 +79,29 @@ class Config:
     n_result: int = 1
     force: bool = False
     db_path: Optional[str] = "~/.local/share/vectorcode/chromadb/"
+    db_log_path: str = "~/.local/share/vectorcode/"
     db_settings: Optional[dict] = None
     chunk_size: int = 2500
     overlap_ratio: float = 0.2
     query_multiplier: int = -1
     query_exclude: list[PathLike] = field(default_factory=list)
-    reranker: Optional[str] = "cross-encoder/ms-marco-MiniLM-L-6-v2"
-    reranker_params: dict[str, Any] = field(default_factory=dict)
+    reranker: Optional[str] = "CrossEncoderReranker"
+    reranker_params: dict[str, Any] = field(default_factory=lambda: {})
     check_item: Optional[str] = None
     use_absolute_path: bool = False
     include: list[QueryInclude] = field(
         default_factory=lambda: [QueryInclude.path, QueryInclude.document]
     )
     hnsw: dict[str, str | int] = field(default_factory=dict)
+    chunk_filters: dict[str, list[str]] = field(default_factory=dict)
+    encoding: str = "utf8"
 
     @classmethod
     async def import_from(cls, config_dict: dict[str, Any]) -> "Config":
         """
         Raise IOError if db_path is not valid.
         """
+        default_config = Config()
         db_path = config_dict.get("db_path")
         host = config_dict.get("host") or "localhost"
         port = config_dict.get("port") or 8000
@@ -94,21 +114,36 @@ class Config:
         return Config(
             **{
                 "embedding_function": config_dict.get(
-                    "embedding_function", "SentenceTransformerEmbeddingFunction"
+                    "embedding_function", default_config.embedding_function
+                ),
+                "embedding_params": config_dict.get(
+                    "embedding_params", default_config.embedding_params
                 ),
-                "embedding_params": config_dict.get("embedding_params", {}),
                 "host": host,
                 "port": port,
                 "db_path": db_path,
-                "chunk_size": config_dict.get("chunk_size", 2500),
-                "overlap_ratio": config_dict.get("overlap_ratio", 0.2),
-                "query_multiplier": config_dict.get("query_multiplier", -1),
-                "reranker": config_dict.get(
-                    "reranker", "cross-encoder/ms-marco-MiniLM-L-6-v2"
+                "db_log_path": os.path.expanduser(
+                    config_dict.get("db_log_path", default_config.db_log_path)
                 ),
-                "reranker_params": config_dict.get("reranker_params", {}),
-                "db_settings": config_dict.get("db_settings", None),
-                "hnsw": config_dict.get("hnsw", {}),
+                "chunk_size": config_dict.get("chunk_size", default_config.chunk_size),
+                "overlap_ratio": config_dict.get(
+                    "overlap_ratio", default_config.overlap_ratio
+                ),
+                "query_multiplier": config_dict.get(
+                    "query_multiplier", default_config.query_multiplier
+                ),
+                "reranker": config_dict.get("reranker", default_config.reranker),
+                "reranker_params": config_dict.get(
+                    "reranker_params", default_config.reranker_params
+                ),
+                "db_settings": config_dict.get(
+                    "db_settings", default_config.db_settings
+                ),
+                "hnsw": config_dict.get("hnsw", default_config.hnsw),
+                "chunk_filters": config_dict.get(
+                    "chunk_filters", default_config.chunk_filters
+                ),
+                "encoding": config_dict.get("encoding", default_config.encoding),
             }
         )
 
@@ -136,17 +171,23 @@ class Config:
 
 def get_cli_parser():
     shared_parser = argparse.ArgumentParser(add_help=False)
-    chunkinng_parser = argparse.ArgumentParser(add_help=False)
-    chunkinng_parser.add_argument(
+    chunking_parser = argparse.ArgumentParser(add_help=False)
+    chunking_parser.add_argument(
         "--overlap", "-o", type=float, help="Ratio of overlaps between chunks."
     )
-    chunkinng_parser.add_argument(
+    chunking_parser.add_argument(
         "-c",
         "--chunk_size",
         type=int,
         default=-1,
         help="Size of chunks (-1 for no chunking).",
     )
+    chunking_parser.add_argument(
+        "--encoding",
+        type=str,
+        default="utf8",
+        help="Encoding used by the files. See https://docs.python.org/3/library/codecs.html#standard-encodings for supported encodings. Use `_auto` for automatic encoding detection.",
+    )
     shared_parser.add_argument(
         "--project_root",
         default=None,
@@ -185,7 +226,7 @@ def get_cli_parser():
 
     vectorise_parser = subparsers.add_parser(
         "vectorise",
-        parents=[shared_parser, chunkinng_parser],
+        parents=[shared_parser, chunking_parser],
         help="Vectorise and send documents to chromadb.",
     )
     vectorise_parser.add_argument(
@@ -198,6 +239,12 @@ def get_cli_parser():
         default=False,
         help="Recursive indexing for directories.",
     )
+    vectorise_parser.add_argument(
+        "--include-hidden",
+        action="store_true",
+        default=False,
+        help="Include hidden files.",
+    )
     vectorise_parser.add_argument(
         "--force",
         "-f",
@@ -208,7 +255,7 @@ def get_cli_parser():
 
     query_parser = subparsers.add_parser(
         "query",
-        parents=[shared_parser, chunkinng_parser],
+        parents=[shared_parser, chunking_parser],
         help="Send query to retrieve documents.",
     )
     query_parser.add_argument("query", nargs="+", help="Query keywords.")
@@ -281,6 +328,15 @@ def get_cli_parser():
         parents=[shared_parser],
         help="Print a list of guidelines intended to be used as system prompts for an LLM.",
     )
+
+    chunks_parser = subparsers.add_parser(
+        "chunks",
+        parents=[shared_parser, chunking_parser],
+        help="Print a JSON array containing chunked text.",
+    )
+    chunks_parser.add_argument(
+        "file_paths", nargs="*", help="Paths to files to be chunked."
+    ).complete = shtab.FILE
     return main_parser
 
 
@@ -290,54 +346,40 @@ async def parse_cli_args(args: Optional[Sequence[str]] = None):
     if main_args.action is None:
         main_args = main_parser.parse_args(["--help"])
 
-    files = []
-    query = None
-    recursive = False
-    number_of_result = 1
-    force = False
-    chunk_size = -1
-    overlap_ratio = 0.2
-    query_multiplier = -1
-    query_exclude = []
-    query_include = ["path", "document"]
-    check_item = None
-    absolute = False
+    configs_items: dict[str, Any] = {
+        "no_stderr": main_args.no_stderr,
+        "action": CliAction(main_args.action),
+        "project_root": main_args.project_root,
+        "pipe": main_args.pipe,
+    }
+
     match main_args.action:
         case "vectorise":
-            files = main_args.file_paths
-            recursive = main_args.recursive
-            force = main_args.force
-            chunk_size = main_args.chunk_size
-            overlap_ratio = main_args.overlap
+            configs_items["files"] = main_args.file_paths
+            configs_items["recursive"] = main_args.recursive
+            configs_items["include_hidden"] = main_args.include_hidden
+            configs_items["force"] = main_args.force
+            configs_items["chunk_size"] = main_args.chunk_size
+            configs_items["overlap_ratio"] = main_args.overlap
+            configs_items["encoding"] = main_args.encoding
         case "query":
-            query = main_args.query
-            number_of_result = main_args.number
-            query_multiplier = main_args.multiplier
-            query_exclude = main_args.exclude
-            absolute = main_args.absolute
-            query_include = main_args.include
+            configs_items["query"] = main_args.query
+            configs_items["n_result"] = main_args.number
+            configs_items["query_multiplier"] = main_args.multiplier
+            configs_items["query_exclude"] = main_args.exclude
+            configs_items["use_absolute_path"] = main_args.absolute
+            configs_items["include"] = [QueryInclude(i) for i in main_args.include]
+            configs_items["encoding"] = main_args.encoding
         case "check":
-            check_item = main_args.check_item
+            configs_items["check_item"] = main_args.check_item
         case "init":
-            force = main_args.force
-    return Config(
-        no_stderr=main_args.no_stderr,
-        action=CliAction(main_args.action),
-        files=files,
-        project_root=main_args.project_root,
-        query=query,
-        recursive=recursive,
-        n_result=number_of_result,
-        pipe=main_args.pipe,
-        force=force,
-        chunk_size=chunk_size,
-        overlap_ratio=overlap_ratio,
-        query_multiplier=query_multiplier,
-        query_exclude=query_exclude,
-        check_item=check_item,
-        use_absolute_path=absolute,
-        include=[QueryInclude(i) for i in query_include],
-    )
+            configs_items["force"] = main_args.force
+        case "chunks":
+            configs_items["files"] = main_args.file_paths
+            configs_items["chunk_size"] = main_args.chunk_size
+            configs_items["overlap_ratio"] = main_args.overlap
+            configs_items["encoding"] = main_args.encoding
+    return Config(**configs_items)
 
 
 def expand_envs_in_dict(d: dict):
@@ -358,10 +400,13 @@ async def load_config_file(path: Optional[PathLike] = None):
     if path is None:
         path = GLOBAL_CONFIG_PATH
     if os.path.isfile(path):
+        logger.debug(f"Loading config from {path}")
         with open(path) as fin:
             config = json.load(fin)
         expand_envs_in_dict(config)
         return await Config.import_from(config)
+    else:
+        logger.warning("Loading default config.")
     return Config()
 
 
@@ -373,9 +418,13 @@ async def find_project_config_dir(start_from: PathLike = "."):
         for anchor in project_root_anchors:
             to_be_checked = os.path.join(current_dir, anchor)
             if os.path.isdir(to_be_checked):
+                logger.debug(f"Found root anchor at {str(to_be_checked)}")
                 return to_be_checked
         parent = current_dir.parent
         if parent.resolve() == current_dir:
+            logger.debug(
+                f"Couldn't find project root after reaching {str(current_dir)}"
+            )
             return
         current_dir = parent.resolve()
 
@@ -419,7 +468,7 @@ def expand_path(path: PathLike, absolute: bool = False) -> PathLike:
 
 
 async def expand_globs(
-    paths: list[PathLike], recursive: bool = False
+    paths: list[PathLike], recursive: bool = False, include_hidden: bool = False
 ) -> list[PathLike]:
     result = set()
     stack = paths
@@ -427,8 +476,77 @@ async def expand_globs(
         curr = stack.pop()
         if os.path.isfile(curr):
             result.add(expand_path(curr))
+        elif "**" in str(curr):
+            stack.extend(
+                glob.glob(str(curr), recursive=True, include_hidden=include_hidden)
+            )
         elif "*" in str(curr):
-            stack.extend(glob.glob(str(curr), recursive=recursive))
+            stack.extend(
+                glob.glob(str(curr), recursive=recursive, include_hidden=include_hidden)
+            )
         elif os.path.isdir(curr) and recursive:
-            stack.extend(glob.glob(os.path.join(curr, "**", "*"), recursive=recursive))
+            stack.extend(
+                glob.glob(
+                    os.path.join(curr, "**", "*"),
+                    recursive=recursive,
+                    include_hidden=include_hidden,
+                )
+            )
     return list(result)
+
+
+def config_logging(
+    name: str = "vectorcode", stdio: bool = True, log_file: bool = False
+):  # pragma: nocover
+    """Configure the logging module. This should be called before a `main` function (CLI, LSP or MCP)."""
+
+    logging.root.handlers = []
+
+    level_from_env = os.environ.get("VECTORCODE_LOG_LEVEL")
+    level = None
+    if level_from_env:
+        level = logging._nameToLevel.get(level_from_env.upper())
+        if level is None:
+            logging.warning(
+                "Invalid log level: %s. Falling back to default levels.", level_from_env
+            )
+
+    handlers = []
+    if level is not None or log_file:
+        log_dir = os.path.expanduser("~/.local/share/vectorcode/logs/")
+        os.makedirs(log_dir, exist_ok=True)
+        # File handler
+        log_file_path = os.path.join(
+            log_dir, f"{name}-{datetime.now().strftime('%Y-%m-%d_%H-%M-%S')}.log"
+        )
+        file_handler = logging.FileHandler(log_file_path)
+        file_handler.setLevel(level or logging.WARN)
+        handlers.append(file_handler)
+        if stdio:
+            atexit.register(lambda: print(f"Saving log to {log_file_path}."))
+
+    if stdio:
+        import colorlog
+
+        # Console handler
+        console_handler = colorlog.StreamHandler(sys.stderr)
+        console_handler.setFormatter(
+            colorlog.ColoredFormatter(
+                fmt="%(log_color)s%(levelname)s%(reset)s: %(name)s : %(message)s",
+                log_colors={
+                    "DEBUG": "cyan",
+                    "INFO": "green",
+                    "WARNING": "yellow",
+                    "ERROR": "red",
+                    "CRITICAL": "bold_red",
+                },
+                reset=True,
+            )
+        )
+        console_handler.setLevel(level or logging.WARN)
+        handlers.append(console_handler)
+
+    logging.basicConfig(
+        handlers=handlers,
+        level=level,
+    )
